
//###########################################################################
//### header and definitions
//###########################################################################

#include <avr/io.h>    // Registernamen fr AVR
#include "timing.h"    // include custom delay functions
#include "AD9958_board.h"    // include some functions from this file

#ifndef BITLENGTH      //bitlength for SPI communication to AD9958
#define BITLENGTH 100  //100us Bitlength = 10kHz SPI to AD9958
#endif

#ifndef SYS_CLK 
#define SYS_CLK 500    // DDS-system clock frequency in MHz
#endif

 //+++ definitions for AD9958 communication +++


#define AD9958_PORT PORTA  

#define SDIO_3 		PA1
#define CS 			PA0
#define SCLK		PA3
#define IO_UPDATE 	PA2
#define DATA		PA5
#define RESET		PA4

//###########################################################################
//### functions
//###########################################################################

void reset_AD9958(void)
{

AD9958_PORT &= ~(1<<RESET); // RESET = 0
delay_us(BITLENGTH);
AD9958_PORT |= (1<<RESET);  // RESET = 1  -->active high
delay_us(BITLENGTH);
AD9958_PORT &= ~(1<<RESET); // RESET = 0

} //reset_AD9958

//###########################################################################

void send_byte_to_AD9958(unsigned char data)
{
 unsigned char count = 0;
 
 for(count=0;count<8;count++)  //send 8 bits = 1 byte
 {
  AD9958_PORT &= ~(1<<SCLK); // clock line = 0
  delay_us(BITLENGTH);
  if((data & 0x80)>>7)  //send MSB first
   {
    AD9958_PORT |= (1<<DATA);  // send 1
   }
  else
   {
    AD9958_PORT &= ~(1<<DATA); //send 0 
   }
  delay_us(BITLENGTH);
  AD9958_PORT |= (1<<SCLK);  // rising edge on clock line transfers data to AD9958
  data = data*2;  //shift for next bit 
 } //for
}//send_byte_to_DDS  

//###########################################################################
           
void update_io_registers(void) //transfers data from DDS buffer register to active registers
{

 AD9958_PORT &= ~(1<<IO_UPDATE);  // IO_UPDATE = 0
 delay_us(BITLENGTH);
 AD9958_PORT |= (1<<IO_UPDATE);  // IO_UPDATE = 1 
 delay_us(BITLENGTH);
 AD9958_PORT &= ~(1<<IO_UPDATE);  // IO_UPDATE = 0

} //update_io_registers

//###########################################################################

void general_AD9958_setup(void)
{
 send_byte_to_AD9958(2);    //instruction byte: write access to register FR2 (0x02)
 send_byte_to_AD9958(32);   //register FR2 High Byte: auto clear phase accumulator
 send_byte_to_AD9958(0);    //register FR2 Low Byte:  keep default settings        
          
 send_byte_to_AD9958(1);    //instruction byte: write access to register FR1 (0x01)
 send_byte_to_AD9958(208);  //register FR1 Byte <23:16> :enable VCO gain control for operation > 255MHz, enable PLL factor 20                                         
 send_byte_to_AD9958(0);    //register FR1 Byte <15:8>  :keep default settings
 send_byte_to_AD9958(0);    //register FR1 Byte <7:0>   :keep default settings

} //general_DDS_setup   

//###########################################################################

void set_fout_ch0(uint32_t frequency)
{
 uint8_t byte1,byte2,byte3,byte4; // 4 bytes for 32bit frequency tuning word

 calc_freq_word(frequency,&byte1,&byte2,&byte3,&byte4);
          
 send_byte_to_AD9958(0);   //instruction byte: write access to register CSR (0x00)
 send_byte_to_AD9958(64); //enable channel 1 , disable channel 0
          
 send_byte_to_AD9958(4);      //instruction byte: write access to register CTW0 (0x04)
 
 send_byte_to_AD9958(byte4);  //the following 4 bytes determine the 32bit frequency tuning word
 send_byte_to_AD9958(byte3);
 send_byte_to_AD9958(byte2);
 send_byte_to_AD9958(byte1);
       
 update_io_registers(); //transfer data from buffer to active registers
 } //set_fout_ch1   

//###########################################################################

void set_fout_ch1(uint32_t frequency)
{
 uint8_t byte1,byte2,byte3,byte4; // 4 bytes for 32bit frequency tuning word

 calc_freq_word(frequency,&byte1,&byte2,&byte3,&byte4);
          
 send_byte_to_AD9958(0);   //instruction byte: write access to register CSR (0x00)
 send_byte_to_AD9958(128); //enable channel 1 , disable channel 0
          
 send_byte_to_AD9958(4);      //instruction byte: write access to register CTW0 (0x04)
 
 send_byte_to_AD9958(byte4);  //the following 4 bytes determine the 32bit frequency tuning word
 send_byte_to_AD9958(byte3);
 send_byte_to_AD9958(byte2);
 send_byte_to_AD9958(byte1);
       
 update_io_registers(); //transfer data from buffer to active registers
 } //set_fout_ch1   


//###########################################################################

void calc_freq_word(uint32_t freq,uint8_t *byte_1,uint8_t *byte_2,uint8_t *byte_3,uint8_t *byte_4)
{
 double fractional_buffer = 0.0;
 uint32_t u32_freq_tuning_word;

  fractional_buffer = (double)freq / SYS_CLK;
  fractional_buffer *= (double)4294.967296;   //constant: 32bit tuning word and sys_clk in[MHz]--> (2^32)/(10^6)=4294.967296
    
  u32_freq_tuning_word = (uint32_t)fractional_buffer;
  
  *byte_4 = (u32_freq_tuning_word>>24);       //most significant 8 bits
  *byte_3 = (u32_freq_tuning_word>>16)&0xff;  //...
  *byte_2 = (u32_freq_tuning_word>>8)&0xff;   //...                                               
  *byte_1 =  u32_freq_tuning_word&0xff;       //least significant 8 bits
} 
